/**
 * Marlin 3D Printer Firmware
 * Copyright (C) 2016 MarlinFirmware [https://github.com/MarlinFirmware/Marlin]
 *
 * Based on Sprinter and grbl.
 * Copyright (C) 2011 Camiel Gubbels / Erik van der Zalm
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
 * M163: Set a single mix factor for a mixing extruder
 *       This is called "weight" by some systems.
 *
 *   S[index]   The channel index to set
 *   P[float]   The mix value
 *
 */
void gcode_M163() {
  const int mix_index = parser.intval('S');
  if (mix_index < MIXING_STEPPERS) {
    float mix_value = parser.floatval('P');
    NOLESS(mix_value, 0.0);
    mixing_factor[mix_index] = RECIPROCAL(mix_value);
  }
}
